@@ -1,7 +1,7 @@
 /*
  * roster.c     -- Local roster implementation
  *
- * Copyright (C) 2005, 2006 Mikael Berthe <bmikael@lists.lilotux.net>
+ * Copyright (C) 2005-2007 Mikael Berthe <mikael@lilotux.net>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -36,7 +36,7 @@ char *straffil[] = { /* Should match enum roster.h */
   "none",
   "owner",
   "admin",
-  "memeber",
+  "member",
   "outcast"
 };
 
@@ -51,6 +51,16 @@ typedef struct {
   enum imrole role;
   enum imaffiliation affil;
   gchar *realjid;       /* for chatrooms, if buddy's real jid is known */
+  guint events;
+#ifdef JEP0022
+  struct jep0022 jep22;
+#endif
+#ifdef JEP0085
+  struct jep0085 jep85;
+#endif
+#ifdef HAVE_GPGME
+  struct pgp_data pgpdata;
+#endif
 } res;
 
 /* This is a private structure type for the roster */
@@ -67,6 +77,9 @@ typedef struct {
   gchar *topic;
   guint inside_room;
 
+  /* on_server is TRUE if the item is present on the server roster */
+  guint on_server;
+
   /* To keep track of last status message */
   gchar *offline_status_message;
 
@@ -80,7 +93,7 @@ typedef struct {
 
 /* ### Variables ### */
 
-static int hide_offline_buddies;
+static guchar display_filter;
 static GSList *groups;
 static GSList *unread_list;
 static GHashTable *unread_jids;
@@ -93,6 +106,9 @@ static roster roster_special;
 void unread_jid_add(const char *jid);
 int  unread_jid_del(const char *jid);
 
+#define DFILTER_ALL     63
+#define DFILTER_ONLINE  62
+
 
 /* ### Initialization ### */
 
@@ -104,17 +120,29 @@ void roster_init(void)
 
 /* ### Resources functions ### */
 
+static inline void free_resource_data(res *p_res)
+{
+  if (!p_res)
+    return;
+  g_free((gchar*)p_res->status_msg);
+  g_free((gchar*)p_res->name);
+  g_free((gchar*)p_res->realjid);
+#ifdef JEP0022
+  g_free(p_res->jep22.last_msgid_sent);
+  g_free(p_res->jep22.last_msgid_rcvd);
+#endif
+#ifdef HAVE_GPGME
+  g_free(p_res->pgpdata.sign_keyid);
+#endif
+  g_free(p_res);
+}
+
 static void free_all_resources(GSList **reslist)
 {
   GSList *lip;
-  res *p_res;
 
-  for ( lip = *reslist; lip ; lip = g_slist_next(lip)) {
-    p_res = (res*)lip->data;
-    g_free((gchar*)p_res->status_msg);
-    g_free((gchar*)p_res->name);
-    g_free((gchar*)p_res->realjid);
-  }
+  for (lip = *reslist; lip ; lip = g_slist_next(lip))
+    free_resource_data((res*)lip->data);
   // Free all nodes but the first (which is static)
   g_slist_free(*reslist);
   *reslist = NULL;
@@ -166,8 +194,14 @@ static res *get_or_add_resource(roster *rost, const char *resname, gchar prio)
 
   for (p = rost->resource; p; p = g_slist_next(p)) {
     res *r = p->data;
-    if (!strcmp(r->name, resname))
+    if (!strcmp(r->name, resname)) {
+      if (prio != r->prio) {
+        r->prio = prio;
+        rost->resource = g_slist_sort(rost->resource,
+                                      (GCompareFunc)&resource_compare_prio);
+      }
       return r;
+    }
   }
 
   // Resource not found
@@ -205,9 +239,7 @@ static void del_resource(roster *rost, const char *resname)
   }
 
   // Free allocations and delete resource node
-  g_free(p_res->name);
-  g_free(p_res->status_msg);
-  g_free(p_res->realjid);
+  free_resource_data(p_res);
   rost->resource = g_slist_delete_link(rost->resource, p_res_elt);
   return;
 }
@@ -215,6 +247,19 @@ static void del_resource(roster *rost, const char *resname)
 
 /* ### Roster functions ### */
 
+static inline void free_roster_user_data(roster *roster_usr)
+{
+  if (!roster_usr)
+    return;
+  g_free((gchar*)roster_usr->jid);
+  g_free((gchar*)roster_usr->name);
+  g_free((gchar*)roster_usr->nickname);
+  g_free((gchar*)roster_usr->topic);
+  g_free((gchar*)roster_usr->offline_status_message);
+  free_all_resources(&roster_usr->resource);
+  g_free(roster_usr);
+}
+
 // Comparison function used to search in the roster (compares jids and types)
 static gint roster_compare_jid_type(roster *a, roster *b) {
   if (! (a->type & b->type))
@@ -240,7 +285,7 @@ static gint roster_compare_name(roster *a, roster *b) {
 GSList *roster_find(const char *jidname, enum findwhat type, guint roster_type)
 {
   GSList *sl_roster_elt = groups;
-  GSList *res;
+  GSList *resource;
   roster sample;
   GCompareFunc comp;
 
@@ -266,8 +311,8 @@ GSList *roster_find(const char *jidname, enum findwhat type, guint roster_type)
       if ((type == namesearch) && !strcmp(jidname, roster_elt->name))
         return sl_roster_elt;
     }
-    res = g_slist_find_custom(roster_elt->list, &sample, comp);
-    if (res) return res;
+    resource = g_slist_find_custom(roster_elt->list, &sample, comp);
+    if (resource) return resource;
     sl_roster_elt = g_slist_next(sl_roster_elt);
   }
   return NULL;
@@ -295,8 +340,9 @@ GSList *roster_add_group(const char *name)
 }
 
 // Returns a pointer to the new user, or existing user with that name
+// Note: if onserver is -1, the flag won't be changed.
 GSList *roster_add_user(const char *jid, const char *name, const char *group,
-                        guint type, enum subscr esub)
+                        guint type, enum subscr esub, gint onserver)
 {
   roster *roster_usr;
   roster *my_group;
@@ -319,12 +365,18 @@ GSList *roster_add_user(const char *jid, const char *name, const char *group,
     // That's an update
     roster_usr = slist->data;
     roster_usr->subscription = esub;
+    if (onserver >= 0)
+      buddy_setonserverflag(slist->data, onserver);
     if (name)
       buddy_setname(slist->data, (char*)name);
     // Let's check if the group name has changed
     oldgroupname = ((roster*)((GSList*)roster_usr->list)->data)->name;
-    if (group && strcmp(oldgroupname, group))
+    if (group && strcmp(oldgroupname, group)) {
       buddy_setgroup(slist->data, (char*)group);
+      // Note: buddy_setgroup() updates the user lists so we cannot
+      // use slist anymore.
+      return roster_find(jid, jidsearch, 0);
+    }
     return slist;
   }
   // #2 add group if necessary
@@ -338,7 +390,7 @@ GSList *roster_add_user(const char *jid, const char *name, const char *group,
     roster_usr->name  = g_strdup(name);
   } else {
     gchar *p, *str = g_strdup(jid);
-    p = strstr(str, "/");
+    p = strchr(str, JID_RESOURCE_SEPARATOR);
     if (p)  *p = '\0';
     roster_usr->name = g_strdup(str);
     g_free(str);
@@ -351,6 +403,8 @@ GSList *roster_add_user(const char *jid, const char *name, const char *group,
   roster_usr->type = type;
   roster_usr->subscription = esub;
   roster_usr->list = slist;    // (my_group SList element)
+  if (onserver == 1)
+    roster_usr->on_server = TRUE;
   // #4 Insert node (sorted)
   my_group->list = g_slist_insert_sorted(my_group->list, roster_usr,
                                          (GCompareFunc)&roster_compare_name);
@@ -378,14 +432,8 @@ void roster_del_user(const char *jid)
   if (roster_usr->flags & ROSTER_FLAG_MSG)
     unread_jid_add(roster_usr->jid);
 
-  // Let's free memory (jid, name, status message)
-  g_free((gchar*)roster_usr->jid);
-  g_free((gchar*)roster_usr->name);
-  g_free((gchar*)roster_usr->nickname);
-  g_free((gchar*)roster_usr->topic);
-  g_free((gchar*)roster_usr->offline_status_message);
-  free_all_resources(&roster_usr->resource);
-  g_free(roster_usr);
+  // Let's free roster_usr memory (jid, name, status message...)
+  free_roster_user_data(roster_usr);
 
   // That's a little complex, we need to dereference twice
   sl_group = ((roster*)sl_user->data)->list;
@@ -421,14 +469,8 @@ void roster_free(void)
       // If there is a pending unread message, keep track of it
       if (roster_usr->flags & ROSTER_FLAG_MSG)
         unread_jid_add(roster_usr->jid);
-      // Free name and jid
-      g_free((gchar*)roster_usr->jid);
-      g_free((gchar*)roster_usr->name);
-      g_free((gchar*)roster_usr->nickname);
-      g_free((gchar*)roster_usr->topic);
-      g_free((gchar*)roster_usr->offline_status_message);
-      free_all_resources(&roster_usr->resource);
-      g_free(roster_usr);
+      // Free roster_usr data (jid, name, status message...)
+      free_roster_user_data(roster_usr);
       sl_usr = g_slist_next(sl_usr);
     }
     // Free group's users list
@@ -466,7 +508,8 @@ void roster_setstatus(const char *jid, const char *resname, gchar prio,
                         ROSTER_TYPE_USER|ROSTER_TYPE_ROOM|ROSTER_TYPE_AGENT);
   // If we can't find it, we add it
   if (sl_user == NULL)
-    sl_user = roster_add_user(jid, NULL, NULL, ROSTER_TYPE_USER, sub_none);
+    sl_user = roster_add_user(jid, NULL, NULL, ROSTER_TYPE_USER,
+                              sub_none, -1);
 
   // If there is no resource name, we can leave now
   if (!resname) return;
@@ -475,7 +518,6 @@ void roster_setstatus(const char *jid, const char *resname, gchar prio,
 
   // New or updated resource
   p_res = get_or_add_resource(roster_usr, resname, prio);
-  p_res->prio = prio;
   p_res->status = bstat;
   if (p_res->status_msg) {
     g_free((gchar*)p_res->status_msg);
@@ -557,7 +599,7 @@ void roster_msg_setflag(const char *jid, guint special, guint value)
                         ROSTER_TYPE_USER|ROSTER_TYPE_ROOM|ROSTER_TYPE_AGENT);
   // If we can't find it, we add it
   if (sl_user == NULL) {
-    sl_user = roster_add_user(jid, NULL, NULL, ROSTER_TYPE_USER, sub_none);
+    sl_user = roster_add_user(jid, NULL, NULL, ROSTER_TYPE_USER, sub_none, -1);
     new_roster_item = TRUE;
   }
 
@@ -600,7 +642,7 @@ void roster_msg_setflag(const char *jid, guint special, guint value)
       // ROSTER_FLAG_MSG should already be set...
   }
 
-  if (buddylist && new_roster_item)
+  if (buddylist && (new_roster_item || !g_list_find(buddylist, roster_usr)))
     buddylist_build();
 }
 
@@ -725,17 +767,31 @@ void roster_unsubscribed(const char *jid)
 // "hide" values: 1=hide 0=show_all -1=invert
 void buddylist_set_hide_offline_buddies(int hide)
 {
-  if (hide < 0)                     // NEG   (invert)
-    hide_offline_buddies = !hide_offline_buddies;
-  else if (hide == 0)               // FALSE (don't hide)
-    hide_offline_buddies = 0;
-  else                              // TRUE  (hide)
-    hide_offline_buddies = 1;
+  if (hide < 0) {               // NEG   (invert)
+    if (display_filter == DFILTER_ALL)
+      display_filter = DFILTER_ONLINE;
+    else
+      display_filter = DFILTER_ALL;
+  } else if (hide == 0) {       // FALSE (don't hide -- andfo_)
+    display_filter = DFILTER_ALL;
+  } else {                      // TRUE  (hide -- andfo)
+    display_filter = DFILTER_ONLINE;
+  }
 }
 
-inline int buddylist_get_hide_offline_buddies(void)
+inline int buddylist_isset_filter(void)
 {
-  return hide_offline_buddies;
+  return (display_filter != DFILTER_ALL);
+}
+
+void buddylist_set_filter(guchar filter)
+{
+  display_filter = filter;
+}
+
+guchar buddylist_get_filter(void)
+{
+  return display_filter;
 }
 
 //  buddylist_build()
@@ -768,17 +824,9 @@ void buddylist_build(void)
   while (sl_roster_elt) {
     GSList *sl_roster_usrelt;
     roster *roster_usrelt;
-    guint pending_group = FALSE;
+    guint pending_group = TRUE;
     roster_elt = (roster*) sl_roster_elt->data;
 
-    // Add the group now unless hide_offline_buddies is set,
-    // in which case we'll add it only if an online buddy belongs to it.
-    // We take care to keep the current_buddy in the list, too.
-    if (!hide_offline_buddies || roster_elt == roster_current_buddy)
-      buddylist = g_list_append(buddylist, roster_elt);
-    else
-      pending_group = TRUE;
-
     shrunk_group = roster_elt->flags & ROSTER_FLAG_HIDE;
 
     sl_roster_usrelt = roster_elt->list;
@@ -786,19 +834,17 @@ void buddylist_build(void)
       roster_usrelt = (roster*) sl_roster_usrelt->data;
 
       // Buddy will be added if either:
-      // - hide_offline_buddies is FALSE
-      // - buddy is not offline
+      // - buddy's status matches the display_filter
       // - buddy has a lock (for example the buddy window is currently open)
       // - buddy has a pending (non-read) message
       // - group isn't hidden (shrunk)
       // - this is the current_buddy
-      if (!hide_offline_buddies || roster_usrelt == roster_current_buddy ||
-          (buddy_getstatus((gpointer)roster_usrelt, NULL) != offline) ||
+      if (roster_usrelt == roster_current_buddy ||
+          display_filter & 1<<buddy_getstatus((gpointer)roster_usrelt, NULL) ||
           (buddy_getflags((gpointer)roster_usrelt) &
-               (ROSTER_FLAG_LOCK | ROSTER_FLAG_MSG))) {
+               (ROSTER_FLAG_LOCK | ROSTER_FLAG_USRLOCK | ROSTER_FLAG_MSG))) {
         // This user should be added.  Maybe the group hasn't been added yet?
-        if (pending_group &&
-            (hide_offline_buddies || roster_usrelt == roster_current_buddy)) {
+        if (pending_group) {
           // It hasn't been done yet
           buddylist = g_list_append(buddylist, roster_elt);
           pending_group = FALSE;
@@ -841,6 +887,8 @@ void buddy_hide_group(gpointer rosterdata, int hide)
 
 const char *buddy_getjid(gpointer rosterdata)
 {
+  if (!rosterdata)
+    return NULL;
   roster *roster_usr = rosterdata;
   return roster_usr->jid;
 }
@@ -848,6 +896,8 @@ const char *buddy_getjid(gpointer rosterdata)
 //  buddy_setgroup()
 // Change the group of current buddy
 //
+// Note: buddy_setgroup() updates the user lists.
+//
 void buddy_setgroup(gpointer rosterdata, char *newgroupname)
 {
   roster *roster_usr = rosterdata;
@@ -1062,6 +1112,57 @@ gchar buddy_getresourceprio(gpointer rosterdata, const char *resname)
   return 0;
 }
 
+guint buddy_resource_getevents(gpointer rosterdata, const char *resname)
+{
+  roster *roster_usr = rosterdata;
+  res *p_res = get_resource(roster_usr, resname);
+  if (p_res)
+    return p_res->events;
+  return ROSTER_EVENT_NONE;
+}
+
+void buddy_resource_setevents(gpointer rosterdata, const char *resname,
+                              guint events)
+{
+  roster *roster_usr = rosterdata;
+  res *p_res = get_resource(roster_usr, resname);
+  if (p_res)
+    p_res->events = events;
+}
+
+struct jep0022 *buddy_resource_jep22(gpointer rosterdata, const char *resname)
+{
+#ifdef JEP0022
+  roster *roster_usr = rosterdata;
+  res *p_res = get_resource(roster_usr, resname);
+  if (p_res)
+    return &p_res->jep22;
+#endif
+  return NULL;
+}
+
+struct jep0085 *buddy_resource_jep85(gpointer rosterdata, const char *resname)
+{
+#ifdef JEP0085
+  roster *roster_usr = rosterdata;
+  res *p_res = get_resource(roster_usr, resname);
+  if (p_res)
+    return &p_res->jep85;
+#endif
+  return NULL;
+}
+
+struct pgp_data *buddy_resource_pgp(gpointer rosterdata, const char *resname)
+{
+#ifdef HAVE_GPGME
+  roster *roster_usr = rosterdata;
+  res *p_res = get_resource(roster_usr, resname);
+  if (p_res)
+    return &p_res->pgpdata;
+#endif
+  return NULL;
+}
+
 enum imrole buddy_getrole(gpointer rosterdata, const char *resname)
 {
   roster *roster_usr = rosterdata;
@@ -1189,11 +1290,25 @@ guint buddy_getflags(gpointer rosterdata)
   return roster_usr->flags;
 }
 
+//  buddy_setonserverflag()
+// Set the on_server flag
+void buddy_setonserverflag(gpointer rosterdata, guint onserver)
+{
+  roster *roster_usr = rosterdata;
+  roster_usr->on_server = onserver;
+}
+
+guint buddy_getonserverflag(gpointer rosterdata)
+{
+  roster *roster_usr = rosterdata;
+  return roster_usr->on_server;
+}
+
 //  buddy_search_jid(jid)
 // Look for a buddy with specified jid.
 // Search begins at buddylist; if no match is found in the the buddylist,
 // return NULL;
-GList *buddy_search_jid(char *jid)
+GList *buddy_search_jid(const char *jid)
 {
   GList *buddy;
   roster *roster_usr;
@@ -1307,22 +1422,32 @@ void foreach_group_member(gpointer groupdata,
 GSList *compl_list(guint type)
 {
   GSList *list = NULL;
-  GList  *buddy = buddylist;
+  GSList *sl_roster_elt = groups;
+  roster *roster_elt;
+  GSList *sl_roster_usrelt;
+  roster *roster_usrelt;
 
-  for ( ; buddy ; buddy = g_list_next(buddy)) {
-    guint btype = buddy_gettype(BUDDATA(buddy));
+  while (sl_roster_elt) {       // group list loop
+    roster_elt = (roster*) sl_roster_elt->data;
+
+    if (roster_elt->type & ROSTER_TYPE_SPECIAL)
+      continue; // Skip special items
 
     if (type == ROSTER_TYPE_GROUP) { // (group names)
-      if (btype == ROSTER_TYPE_GROUP) {
-        const char *bname = buddy_getname(BUDDATA(buddy));
-        if ((bname) && (*bname))
-          list = g_slist_append(list, from_utf8(bname));
-      }
+      if (roster_elt->name && *(roster_elt->name))
+        list = g_slist_append(list, from_utf8(roster_elt->name));
     } else { // ROSTER_TYPE_USER (jid) (or agent, or chatroom...)
-        const char *bjid = buddy_getjid(BUDDATA(buddy));
-        if (bjid)
-          list = g_slist_append(list, from_utf8(bjid));
+      sl_roster_usrelt = roster_elt->list;
+      while (sl_roster_usrelt) {  // user list loop
+        roster_usrelt = (roster*) sl_roster_usrelt->data;
+
+        if (roster_usrelt->jid)
+          list = g_slist_append(list, from_utf8(roster_usrelt->jid));
+
+        sl_roster_usrelt = g_slist_next(sl_roster_usrelt);
+      }
     }
+    sl_roster_elt = g_slist_next(sl_roster_elt);
   }
 
   return list;
