@@ -14,22 +14,46 @@
 #define SETTINGS_TYPE_OPTION    1
 #define SETTINGS_TYPE_ALIAS     2
 #define SETTINGS_TYPE_BINDING   3
+#ifdef HAVE_LIBOTR
+#define SETTINGS_TYPE_OTR       4
+#endif
+
+#define COMMAND_CHAR    '/'
+#define COMMAND_CHARSTR "/"
 
 #define settings_opt_get(k)     settings_get(SETTINGS_TYPE_OPTION, k)
 #define settings_opt_get_int(k) settings_get_int(SETTINGS_TYPE_OPTION, k)
 
-int     cfg_read_file(char *filename);
-guint   parse_assigment(gchar *assignment,
-                        const gchar **pkey, const gchar **pval);
+#define mkcmdstr(cmd) COMMAND_CHARSTR cmd
+
+void    settings_init(void);
+int     cfg_read_file(char *filename, guint mainfile);
+guint   parse_assigment(gchar *assignment, gchar **pkey, gchar **pval);
 void    settings_set(guint type, const gchar *key, const gchar *value);
 void    settings_del(guint type, const gchar *key);
 const gchar *settings_get(guint type, const gchar *key);
 int     settings_get_int(guint type, const gchar *key);
 const gchar *settings_get_status_msg(enum imstatus status);
 void    settings_foreach(guint type,
-                         void (*pfunc)(void *param, char *k, char *v),
+                         void (*pfunc)(char *k, char *v, void *param),
                          void *param);
 
+void    settings_pgp_setdisabled(const char *bjid, guint value);
+guint   settings_pgp_getdisabled(const char *bjid);
+void    settings_pgp_setforce(const char *bjid, guint value);
+guint   settings_pgp_getforce(const char *bjid);
+void    settings_pgp_setkeyid(const char *bjid, const char *keyid);
+const char *settings_pgp_getkeyid(const char *bjid);
+
+#ifdef HAVE_LIBOTR
+guint   settings_otr_getpolicy(const char *bjid);
+void    settings_otr_setpolicy(const char *bjid, guint value);
+#endif
+
+guint get_max_history_blocks(void);
+
+char *default_muc_nickname(void);
+
 const gchar *isbound(int key);
 
 #endif /* __SETTINGS_H__ */
